@@ -52,7 +52,7 @@ Glib::ustring encoding_display(const Glib::ustring& name, const Glib::ustring& c
   if(charset.empty())
     return name;
   else
-    return name + " (" + charset + ")";
+    return name + " (" + charset + ')';
 }
 
 } //anonymous namespace
@@ -88,7 +88,7 @@ Dialog_Import_CSV::Dialog_Import_CSV(BaseObjectType* cobject, const Glib::RefPtr
   m_encoding_model->append();
 
   const FileEncodings::type_list_encodings list_encodings =  FileEncodings::get_list_of_encodings();
-  for(FileEncodings::type_list_encodings::const_iterator encodings_iter = list_encodings.begin(); encodings_iter  != list_encodings.end(); encodings_iter ++)
+  for(FileEncodings::type_list_encodings::const_iterator encodings_iter = list_encodings.begin(); encodings_iter  != list_encodings.end(); ++encodings_iter)
   {
     const FileEncodings::Encoding encoding = *encodings_iter;
     if(encoding.get_name().empty())
@@ -119,6 +119,7 @@ Dialog_Import_CSV::Dialog_Import_CSV(BaseObjectType* cobject, const Glib::RefPtr
   m_parser->signal_line_scanned().connect(sigc::mem_fun(*this, &Dialog_Import_CSV::on_parser_line_scanned));
   m_parser->signal_state_changed().connect(sigc::mem_fun(*this, &Dialog_Import_CSV::on_parser_state_changed));
 
+  m_first_line_as_title->set_active(false);
   m_first_line_as_title->signal_toggled().connect(sigc::mem_fun(*this, &Dialog_Import_CSV::on_first_line_as_title_toggled));
   m_sample_rows->signal_changed().connect(sigc::mem_fun(*this, &Dialog_Import_CSV::on_sample_rows_changed));
 
@@ -725,8 +726,23 @@ void Dialog_Import_CSV::validate_primary_key()
 
 void Dialog_Import_CSV::on_parser_file_read_error(const Glib::ustring& error_message)
 {
+  std::string filename;
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  try
+  {
+    filename = Glib::filename_from_uri(m_file_uri);
+  }
+  catch(const Glib::ConvertError& ex)
+  {
+    std::cerr << "Glib::filename_from_uri() failed: " << ex.what() << std::endl;
+  }
+#else
+  std::auto_ptr<Glib::Error> error;
+  filename = Glib::filename_from_uri(m_file_uri, error);
+#endif
+  
   show_error_dialog(_("Could Not Open file"),
-    Glib::ustring::compose(_("The file at \"%1\" could not be opened: %2"), Glib::filename_from_uri(m_file_uri), error_message) );
+    Glib::ustring::compose(_("The file at \"%1\" could not be opened: %2"), filename, error_message) );
 }
 
 void Dialog_Import_CSV::on_parser_have_display_name(const Glib::ustring& display_name)
