@@ -17,8 +17,14 @@
  * Returns: 0 on success, -1 on error.
  */
 int
-pygda_value_from_pyobject(GValue *boxed, PyObject *input)
+glom_pygda_value_from_pyobject(GValue *boxed, PyObject *input)
 {
+    // We shouldn't need to call PyDateTime_IMPORT again, 
+    // after already doing it in libglom_init(),
+    // but PyDate_Check crashes (with valgrind warnings) if we don't.
+    PyDateTime_IMPORT; //A macro, needed to use PyDate_Check(), PyDateTime_Check(), etc.
+    g_assert(PyDateTimeAPI); //This should have been set by the PyDateTime_IMPORT macro.
+
     /* Use an appropriate gda_value_set_*() function.
        We can not know what GValue type is actually wanted, so
        we must still have the get_*() functions in the python API.
@@ -55,12 +61,13 @@ pygda_value_from_pyobject(GValue *boxed, PyObject *input)
          gda.timezone = 0;
          gda_value_set_timestamp (boxed, &gda);
      } else if (PyDate_Check (input)) {
-         GDate gda;
-         gda.year = PyDateTime_GET_YEAR(input);
-         gda.month = PyDateTime_GET_MONTH(input);
-         gda.day = PyDateTime_GET_DAY(input);
+         GDate *gda = g_date_new_dmy(
+           PyDateTime_GET_DAY(input),
+           PyDateTime_GET_MONTH(input),
+           PyDateTime_GET_YEAR(input) );
          g_value_init (boxed, G_TYPE_DATE);
-         g_value_set_boxed(boxed, &gda);
+         g_value_set_boxed(boxed, gda);
+         g_date_free(gda);
      } else if (PyTime_Check (input)) {
          GdaTime gda;
          gda.hour = PyDateTime_TIME_GET_HOUR(input);
@@ -88,7 +95,7 @@ pygda_value_from_pyobject(GValue *boxed, PyObject *input)
  * Returns: a PyObject representing the value.
  */
 PyObject *
-pygda_value_as_pyobject(const GValue *boxed, gboolean copy_boxed)
+glom_pygda_value_as_pyobject(const GValue *boxed, gboolean copy_boxed)
 {
     copy_boxed = FALSE; //Just to avoid the "unused paramter" and "parameter name omitted" warnings.
 
