@@ -134,17 +134,24 @@ Glib::ustring Utils::trim_whitespace(const Glib::ustring& text)
 
 Glib::ustring Utils::string_replace(const Glib::ustring& src, const Glib::ustring search_for, const Glib::ustring& replace_with)
 {
+  if(search_for.empty())
+  {
+    std::cerr << "Utils::string_replace(): search_for was empty." << std::endl;
+    return src;
+  }
+
   //std::cout << "debug: Utils::string_replace(): src=" << src << ", search_for=" << search_for << ", replace_with=" << replace_with << std::endl;
 
   std::string result = src;
 
   std::string::size_type pos = 0;
-  std::string::size_type len_search = search_for.size();
-  std::string::size_type len_replace = replace_with.size();
+  const std::string::size_type len_search = search_for.size();
+  const std::string::size_type len_replace = replace_with.size();
 
   std::string::size_type pos_after_prev = 0;
   while((pos = result.find(search_for, pos_after_prev)) != std::string::npos)
   {
+    //std::cout << "  debug: pos=" << pos << ", found=" << search_for << ", in string: " << result.substr(pos_after_prev, 20) << std::endl;
     //std::cout << "  debug: before: result =" << result << ", pos_after_prev=pos_after_prev" << std::endl;
     result.replace(pos, len_search, replace_with);
     //std::cout << "  after: before: result = result" << std::endl;
@@ -269,13 +276,13 @@ Glib::ustring Utils::build_sql_select_fields_to_get(const Glib::ustring& table_n
 
     //Add, for instance, "SUM(":
     if(is_summary)
-      one_sql_part += fieldsummary->get_summary_type_sql() + "(";
+      one_sql_part += fieldsummary->get_summary_type_sql() + '(';
 
     one_sql_part += layout_item->get_sql_name(table_name);
 
     //Close the summary bracket if necessary.
     if(is_summary)
-      one_sql_part +=  ")";
+      one_sql_part +=  ')';
 
     //Append it to the big string of fields:
     if(!one_sql_part.empty())
@@ -328,16 +335,16 @@ Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& tab
   //Build the whole SQL statement:
   Glib::ustring result = 
     "SELECT " + sql_part_fields +
-    " FROM \"" + table_name + "\"";
+    " FROM \"" + table_name + '\"';
 
   if(!sql_part_from.empty())
-    result += ("," + sql_part_from);
+    result += (',' + sql_part_from);
 
   if(!extra_join.empty())
-    sql_part_leftouterjoin += (" " + extra_join + " ");
+    sql_part_leftouterjoin += (' ' + extra_join + ' ');
 
   if(!sql_part_leftouterjoin.empty())
-    result += (" " + sql_part_leftouterjoin);
+    result += (' ' + sql_part_leftouterjoin);
 
 
   //Add the WHERE clause:
@@ -347,7 +354,7 @@ Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& tab
   //Extra GROUP_BY clause for doubly-related records. This must be before the ORDER BY sort clause:
   if(!extra_group_by.empty())
   {
-    result += (" " + extra_group_by + " ");
+    result += (' ' + extra_group_by + ' ');
   }
 
   //Sort clause:
@@ -395,7 +402,7 @@ Glib::ustring Utils::build_sql_select_with_key(const Glib::ustring& table_name,
   if(!Conversions::value_is_empty(key_value)) //If there is a record to show:
   {
     //TODO: Use a SQL parameter instead of using sql():
-    const Glib::ustring where_clause = "\"" + table_name + "\".\"" + key_field->get_name() + "\" = " + key_field->sql(key_value);
+    const Glib::ustring where_clause = '\"' + table_name + "\".\"" + key_field->get_name() + "\" = " + key_field->sql(key_value);
     return Utils::build_sql_select_with_where_clause(table_name, fieldsToGet, where_clause);
   }
 
@@ -423,14 +430,14 @@ Utils::type_list_values_with_second Utils::get_choice_values(const sharedptr<con
   }
 
   const bool with_second = !choice_second.empty();
-  const Glib::ustring sql_second = "\"" + to_table + "\".\"" + choice_second + "\"";
+  const Glib::ustring sql_second = "\"" + to_table + "\".\"" + choice_second + '\"';
 
   //Get possible values from database, sorted by the first column.
-  Glib::ustring sql_query = "SELECT \"" + to_table + "\".\"" + choice_field + "\"";
+  Glib::ustring sql_query = "SELECT \"" + to_table + "\".\"" + choice_field + '\"';
   if(with_second)
     sql_query += ", " + sql_second;
 
-  sql_query += " FROM \"" + choice_relationship->get_to_table() + "\" ORDER BY \"" + to_table + "\".\"" + choice_field + "\"";
+  sql_query += " FROM \"" + choice_relationship->get_to_table() + "\" ORDER BY \"" + to_table + "\".\"" + choice_field + '\"';
 
   //std::cout << "debug: get_choice_values(): query: " << sql_query << std::endl;
   //Connect to database:
@@ -472,8 +479,9 @@ Utils::type_list_values_with_second Utils::get_choice_values(const sharedptr<con
 
       if(with_second)
         itempair.second = datamodel->get_value_at(1, row, error);
-      list_values.push_back(itempair);
-#endif      
+#endif
+
+      list_values.push_back(itempair);      
     }
   }
   else
@@ -514,14 +522,14 @@ Glib::ustring Utils::locale_simplify(const Glib::ustring& locale_id)
   Glib::ustring result = locale_id;
 
   //Get everything before the .:
-  Glib::ustring::size_type posDot = locale_id.find(".");
+  Glib::ustring::size_type posDot = locale_id.find('.');
   if(posDot != Glib::ustring::npos)
   {
     result = result.substr(0, posDot);
   }
 
   //Get everything before the @:
-  const Glib::ustring::size_type posAt = locale_id.find("@");
+  const Glib::ustring::size_type posAt = locale_id.find('@');
   if(posAt != Glib::ustring::npos)
   {
     result = result.substr(0, posAt);
@@ -534,7 +542,7 @@ Glib::ustring Utils::locale_language_id(const Glib::ustring& locale_id)
 {
   Glib::ustring result;
 
-  const Glib::ustring::size_type posUnderscore = locale_id.find("_");
+  const Glib::ustring::size_type posUnderscore = locale_id.find('_');
   if(posUnderscore != Glib::ustring::npos)
   {
     result = locale_id.substr(0, posUnderscore);
