@@ -1,6 +1,6 @@
 /* Glom
  *
- * Copyright (C) 2001-2004 Murray Cumming
+ * Copyright (C) 2001-2009 Murray Cumming
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -37,83 +37,67 @@ namespace Glom
 {
 
 Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
-: m_HBox(false, Utils::DEFAULT_SPACING_SMALL),
-  m_HBox_Sidebar (false, Utils::DEFAULT_SPACING_SMALL),
+: m_hbox_content(false, Utils::DEFAULT_SPACING_SMALL),
+#ifndef GLOM_ENABLE_MAEMO
+  m_hbox_buttons(false, Utils::DEFAULT_SPACING_SMALL),
   m_Button_New(Gtk::Stock::ADD),
   m_Button_Del(Gtk::Stock::DELETE),
   m_Button_Nav_First(Gtk::Stock::GOTO_FIRST),
   m_Button_Nav_Prev(Gtk::Stock::GO_BACK),
   m_Button_Nav_Next(Gtk::Stock::GO_FORWARD),
   m_Button_Nav_Last(Gtk::Stock::GOTO_LAST),
+#endif //GLOM_ENABLE_MAEMO
   m_bDoNotRefreshRelated(false),
   m_ignore_signals(true)
 #ifndef GLOM_ENABLE_CLIENT_ONLY
-  ,m_design_mode(false)
+  , m_design_mode(false)
 #endif
 {
   m_layout_name = "details";
 
-  m_Button_New.set_tooltip_text(_("Create a new record."));
-  m_Button_Del.set_tooltip_text(_("Remove this record."));
-  m_Button_Nav_First.set_tooltip_text(_("View the first record in the list."));
-  m_Button_Nav_Prev.set_tooltip_text(_("View the previous record in the list."));
-  m_Button_Nav_Next.set_tooltip_text(_("View the next record in the list."));
-  m_Button_Nav_Last.set_tooltip_text(_("View the last record in the list."));
-  
-
   add_view(&m_FlowTable); //Allow this to access the document too.
 
   m_FlowTable.set_columns_count(1); //Sub-groups will have multiple columns (by default, there is one sub-group, with 2 columns).
-  m_FlowTable.set_padding(Utils::DEFAULT_SPACING_SMALL);
 
+  #ifndef GLOM_ENABLE_MAEMO
+  m_FlowTable.set_column_padding(Utils::DEFAULT_SPACING_SMALL); //The default anyway.
+  m_FlowTable.set_row_padding(Utils::DEFAULT_SPACING_SMALL); //The default anyway.
+  #else
+  m_FlowTable.set_row_padding(0); //The hildon buttons and entries have their own default padding.
+  m_FlowTable.set_column_padding(HILDON_MARGIN_DOUBLE); //As per the UI specs.
+  #endif
+  
   //m_strHint = _("When you change the data in a field the database is updated immediately.\n Click [New] to add a new record.\n Leave automatic ID fields empty - they will be filled for you.");
 
 
   //m_ScrolledWindow.set_border_width(Utils::DEFAULT_SPACING_SMALL);
 #ifdef GLOM_ENABLE_MAEMO
-  m_ScrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC); /* Allow horizontal scrolling in maemo because the screen is rather small and there might be some database UIs that don't fit horizontally. Such a UI may be concidered non-maemo-friendly, but it can still be fully viewed this way. */
+  // Allow horizontal scrolling in maemo because the screen is rather small and 
+  // there might be some database UIs that don't fit horizontally. Such a UI may 
+  // be considered non-maemo-friendly, but it can still be fully viewed this way.
+  // TODO: Prevent the need for horizontal scrolling.
+  m_ScrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
 #else
-  m_ScrolledWindow.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC); /* Allow vertical scrolling, but never scroll horizontally. */
+  // Allow vertical scrolling, but never scroll horizontally:
+  m_ScrolledWindow.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC); 
 #endif
   m_ScrolledWindow.set_shadow_type(Gtk::SHADOW_NONE); //SHADOW_IN is Recommended by the GNOME HIG, but looks odd.
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
-  m_HBox_Sidebar.pack_start(m_Dragbar, Gtk::PACK_SHRINK);
+  m_hbox_content.pack_start(m_Dragbar, Gtk::PACK_SHRINK);
   m_Dragbar.hide();
 #endif
 
-  m_HBox_Sidebar.pack_start(m_ScrolledWindow);
+  m_hbox_content.pack_start(m_ScrolledWindow);
   m_ScrolledWindow.add(m_FlowTable);
   // The FlowTable does not support native scrolling, so gtkmm adds it to a
   // viewport first that also has some shadow we do not want.
   Gtk::Viewport* viewport = dynamic_cast<Gtk::Viewport*>(m_FlowTable.get_parent());
-  if(viewport) viewport->set_shadow_type(Gtk::SHADOW_NONE);
+  if(viewport)
+    viewport->set_shadow_type(Gtk::SHADOW_NONE);
 
-  //Add or delete record:
-  m_HBox.pack_start(m_Button_New, Gtk::PACK_SHRINK);
-  m_HBox.pack_start(m_Button_Del,  Gtk::PACK_SHRINK);
 
-   //Link buttons to handlers:
-  m_Button_New.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_new));
-  m_Button_Del.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_del));
-
-  //Navigation:
-  if(bWithNavButtons)
-  {
-    m_HBox.pack_end(m_Button_Nav_Last, Gtk::PACK_SHRINK);
-    m_HBox.pack_end(m_Button_Nav_Next, Gtk::PACK_SHRINK);
-    m_HBox.pack_end(m_Button_Nav_Prev, Gtk::PACK_SHRINK);
-    m_HBox.pack_end(m_Button_Nav_First, Gtk::PACK_SHRINK);
-  }
-
-  //Link buttons to handlers:
-  m_Button_Nav_First.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_nav_first));
-  m_Button_Nav_Prev.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_nav_prev));
-  m_Button_Nav_Next.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_nav_next));
-  m_Button_Nav_Last.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_nav_last));
-
-  pack_start(m_HBox_Sidebar);
-  pack_start(m_HBox, Gtk::PACK_SHRINK);
+  pack_start(m_hbox_content);
 
   m_FlowTable.signal_field_edited().connect( sigc::mem_fun(*this,  &Box_Data_Details::on_flowtable_field_edited) );
   m_FlowTable.signal_field_open_details_requested().connect( sigc::mem_fun(*this,  &Box_Data_Details::on_flowtable_field_open_details_requested) );
@@ -130,6 +114,41 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
 
   m_FlowTable.signal_script_button_clicked().connect( sigc::mem_fun(*this, &Box_Data_Details::on_flowtable_script_button_clicked) );
 
+
+#ifndef GLOM_ENABLE_MAEMO
+  m_Button_New.set_tooltip_text(_("Create a new record."));
+  m_Button_Del.set_tooltip_text(_("Remove this record."));
+  m_Button_Nav_First.set_tooltip_text(_("View the first record in the list."));
+  m_Button_Nav_Prev.set_tooltip_text(_("View the previous record in the list."));
+  m_Button_Nav_Next.set_tooltip_text(_("View the next record in the list."));
+  m_Button_Nav_Last.set_tooltip_text(_("View the last record in the list."));
+
+  //Add or delete record:
+  m_hbox_buttons.pack_start(m_Button_New, Gtk::PACK_SHRINK);
+  m_hbox_buttons.pack_start(m_Button_Del,  Gtk::PACK_SHRINK);
+
+  //Link buttons to handlers:
+  m_Button_New.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_new));
+  m_Button_Del.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_del));
+
+  //Navigation:
+  if(bWithNavButtons)
+  {
+    m_hbox_buttons.pack_end(m_Button_Nav_Last, Gtk::PACK_SHRINK);
+    m_hbox_buttons.pack_end(m_Button_Nav_Next, Gtk::PACK_SHRINK);
+    m_hbox_buttons.pack_end(m_Button_Nav_Prev, Gtk::PACK_SHRINK);
+    m_hbox_buttons.pack_end(m_Button_Nav_First, Gtk::PACK_SHRINK);
+  }
+
+  //Link buttons to handlers:
+  m_Button_Nav_First.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_nav_first));
+  m_Button_Nav_Prev.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_nav_prev));
+  m_Button_Nav_Next.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_nav_next));
+  m_Button_Nav_Last.signal_clicked().connect(sigc::mem_fun(*this, &Box_Data_Details::on_button_nav_last));
+
+  pack_start(m_hbox_buttons, Gtk::PACK_SHRINK);
+#endif //GLOM_ENABLE_MAEMO
+
   m_ignore_signals = false;
 }
 
@@ -285,8 +304,10 @@ bool Box_Data_Details::fill_from_database()
       Privileges table_privs = Privs::get_current_privs(m_table_name);
 
       //Enable/Disable record creation and deletion:
+      #ifndef GLOM_ENABLE_MAEMO
       m_Button_New.set_sensitive(table_privs.m_create);
       m_Button_Del.set_sensitive(table_privs.m_delete);
+      #endif //GLOM_ENABLE_MAEMO
 
       if(table_privs.m_view)
       {
@@ -318,7 +339,7 @@ bool Box_Data_Details::fill_from_database()
             if(element->get_name() != layout_item->get_name())
               continue;
             
-            //Compare the relationship and related relationshp:
+            //Compare the relationship and related relationship:
             sharedptr<const UsesRelationship> uses_a = layout_item;
             sharedptr<const UsesRelationship> uses_b = element;
             if(*uses_a == *uses_b)
@@ -445,6 +466,7 @@ void Box_Data_Details::on_button_new()
   }
 }
 
+#ifndef GLOM_ENABLE_MAEMO
 void Box_Data_Details::on_button_del()
 {
   if( Conversions::value_is_empty(get_primary_key_value_selected()) )
@@ -494,6 +516,7 @@ void Box_Data_Details::on_button_nav_last()
   if(confirm_discard_unstored_data())
     signal_nav_last().emit();
 }
+#endif //GLOM_ENABLE_MAEMO
 
 Gnome::Gda::Value Box_Data_Details::get_entered_field_data(const sharedptr<const LayoutItem_Field>& field) const
 {
@@ -580,6 +603,7 @@ void Box_Data_Details::on_related_record_added(Gnome::Gda::Value /* strKeyValue
   m_bDoNotRefreshRelated = bDoNotRefreshRelated;
 }
 
+#ifndef GLOM_ENABLE_MAEMO
 Box_Data_Details::type_signal_void Box_Data_Details::signal_nav_first()
 {
   return m_signal_nav_first;
@@ -604,6 +628,7 @@ Box_Data_Details::type_signal_record_deleted Box_Data_Details::signal_record_del
 {
   return m_signal_record_deleted; 
 }
+#endif //GLOM_ENABLE_MAEMO
 
 Box_Data_Details::type_signal_requested_related_details Box_Data_Details::signal_requested_related_details()
 {
@@ -691,8 +716,10 @@ void Box_Data_Details::on_flowtable_script_button_clicked(const sharedptr<const
     }
     else
     {
+      #ifndef GLOM_ENABLE_MAEMO
       //Tell the parent to do something appropriate, such as show another record:
       signal_record_deleted().emit(primary_key_value);
+      #endif
     }
   }
 }
@@ -817,7 +844,7 @@ void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutIte
         /*
         bool bIsForeignKey = false;
         Document::type_vec_relationships vecRelationships = get_document()->get_relationships(m_table_name);
-        for(Document::type_vec_relationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); iter++)
+        for(Document::type_vec_relationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); ++iter)
         {
           const Relationship& relationship = *iter;
 
@@ -1022,7 +1049,7 @@ Dialog_Layout* Box_Data_Details::create_layout_dialog() const
     return dialog;
   }
 
-  return NULL;
+  return 0;
 }
 
 void Box_Data_Details::prepare_layout_dialog(Dialog_Layout* dialog)
@@ -1041,4 +1068,11 @@ void Box_Data_Details::show_layout_toolbar(bool show)
 
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
+#ifdef GLOM_ENABLE_MAEMO
+void Box_Data_Details::do_new_record()
+{
+  on_button_new();
+}
+#endif
+
 } //namespace Glom
